<?php
/**
 * The working horse for building theme settings. Simplifies the life by mapping theme options
 * with appropriate CSS selectors.
 *
 * There are two classes - one for mapping and another, extension over the base class. The last one
 * adds all the settings together with default values.
 *
 * @since Ventcamp 1.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

//require_once "customizer.class.php";

/**
 * Class Theme_Settings_Builder. Base class for working with customizer.
 */
class Theme_Settings_Builder {

	/**
	 * This prefix is being added before every setting.
	 */
	protected $settings_prefix;

	/**
	 * The set of default settings
	 */
	protected $default_settings;

	/**
	 * Maps of options with properties, available fonts and colors
	 */
	protected $map = array(), $fonts = array(), $colors = array();

	/**
	 * Define defaults, call the `register` method, add css to head.
	 *
	 * @param string $settings_prefix Prefix which is being added to settings
	 * @param array $default_settings Default settings
	 */
	public function create( $settings_prefix = '', $default_settings = array() ) {
		// Assign prefix and default settings (only if they were not defined in child class)
		$this->settings_prefix = $this->settings_prefix ? $this->settings_prefix : $settings_prefix;
		$this->default_settings = $this->default_settings ? $this->default_settings : $default_settings;

		// Register new customizer elements
		add_action( 'customize_register', array( $this, 'register_theme_settings'), 15 );

		// Add inline CSS to the header
		add_action( 'wp_head', array( $this, 'header_css' ) );
	}

	/**
	 * Register layout, colors, color schemes and font settings.
	 *
	 * @param WP_Customize_Manager $wp_customize An instance of WP_Customize_Manager class
	 */
	public function register_theme_settings( $wp_customize ) {
		// Add various settings
		$this->layout( $wp_customize );
		$this->color_scheme( $wp_customize );
		$this->add_color_settings( $wp_customize );
		$this->fonts( $wp_customize );

		// Add customizer JS transport (bindings for live update)
		if ( $wp_customize->is_preview() && !is_admin() ) {
			add_action( 'wp_footer', array( $this, 'footer_scripts' ), 25 );
		}
	}

	/**
	 * Add layout setting to customizer.
	 *
	 * @param WP_Customize_Manager $wp_customize An instance of WP_Customize_Manager class
	 */
	protected function layout( $wp_customize ) {
		// Add layout section
		$wp_customize->add_section( 'ventcamp_lite_layout', array(
			'title'    => __( 'Site Layout', 'ventcamp-lite' ),
			'priority' => 150,
		) );

		// Add settings for site layout
		$wp_customize->add_setting( $this->settings_prefix . '[site_layout]', array(
			'default'    => '',
			'type'       => 'option',
			'capability' => 'edit_theme_options',
			'sanitize_callback'  => 'esc_attr',
		) );

		// Add checkbox control with `Boxed layout` label
		$wp_customize->add_control( $this->settings_prefix . '_site_layout', array(
			'label'      => __( 'Boxed layout', 'ventcamp-lite' ),
			'section'    => 'ventcamp_lite_layout',
			'settings'   => $this->settings_prefix . '[site_layout]',
			'type'       => 'checkbox',
			'std'        => '0'
		) );
	}

	/**
	 * Add color schemes setting to customizer.
	 *
	 * @param WP_Customize_Manager $wp_customize An instance of WP_Customize_Manager class
	 */
	protected function color_scheme( $wp_customize ) {
		/*
		 * The color schemes can be defined this way:
		 *
		 * add_theme_support( 'ventcamp-style-selector', array(
		 *	 'your-theme-blue'   => 'Your Theme Blue',
		 *	 'your-theme-orange' => 'Your Theme Orange'
		 * ) );
		 *
		 * Exit if custom schemes are not defined.
		 */
		if ( !current_theme_supports( 'ventcamp-style-selector' ) ) {
			return;
		}

		// Add color scheme section
		$wp_customize->add_section( 'color_scheme', array(
			'title'    => 'Color Scheme',
			'priority' => 150,
		) );

		// Add settings for color scheme
		$wp_customize->add_setting( $this->settings_prefix . '[style_selection]', array(
			'default'  => '',
			'type'     => 'option',
			'sanitize_callback'  => 'esc_attr',
		) );

		// Add color style dropdown together with label
		$wp_customize->add_control( $this->settings_prefix . '_style_selection', array(
			'label'    => __( 'Select Color Style', 'ventcamp-lite' ),
			'section'  => 'color_scheme',
			'settings' => $this->settings_prefix . '[style_selection]',
			'type'     => 'select',
			'choices'  => array_merge(
				array( '' => __( 'Default', 'ventcamp-lite' ) ),
				array_shift( get_theme_support( 'ventcamp-style-selector' ) )
			),
		) );
	}

	/**
	 * Add colors setting to customizer.
	 *
	 * @param WP_Customize_Manager $wp_customize An instance of WP_Customize_Manager class
	 */
	protected function add_color_settings( $wp_customize ) {
		// Only if `colors` is defined in default settings
		if ( !isset( $this->default_settings['colors'] ) ) {
			return;
		}

		// Remove default colors sections
		$wp_customize->remove_section( 'colors' );
		// Add colors section
		$wp_customize->add_section( 'colors', array(
			'title'    => __( 'Colors', 'ventcamp-lite' ),
			'priority' => 160,
		) );

		// Loop through the colors setting and add settings and controls
		foreach ( $this->default_settings['colors'] as $key => $data ) {
			// Add color setting
			$wp_customize->add_setting( $this->get_field_name( $key, 'colors' ), array(
				'default'   => $data['default'],
				'type'      => 'option',
				'transport' => 'postMessage',
				'sanitize_callback'  => 'esc_attr',
			) );

			// Add colorpicker together with label
			$wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, $this->get_field_name( $key, 'colors' ), array(
				'label'    => $data['label'],
				'section'  => 'colors',
				'settings' => $this->get_field_name( $key, 'colors' ),
			) ) );
		}
	}

	/**
	 * Add fonts setting to customizer.
	 *
	 * @param WP_Customize_Manager $wp_customize An instance of WP_Customize_Manager class
	 */
	protected function fonts( $wp_customize ) {
		// Only if `fonts` is defined in default settings
		if ( !isset( $this->default_settings['fonts'] ) ) {
			return;
		}

		// Add fonts section
		$wp_customize->add_section( 'fonts', array(
			'title'    => __( 'Fonts', 'ventcamp-lite' ),
			'priority' => 160,
		) );

		// Loop through the fonts setting and add settings and controls
		foreach ( $this->default_settings['fonts'] as $key => $data ) {
			// Add font setting
			$wp_customize->add_setting( $this->get_field_name( $key, 'fonts' ), array(
				'default'   => $data['default'],
				'type'      => 'option',
				'sanitize_callback'  => 'esc_attr',
			) );

			// Add dropdown with available fonts
			$wp_customize->add_control( 'fonts-' . $key, array(
				'label'    => $data['label'],
				'section'  => 'fonts',
				'settings' => $this->get_field_name( $key, 'fonts' ),
				'type'     => 'select',
				'choices'  => array_flip( $this->fonts ),
			) );
		}
	}

	/**
	 * Get values from settings and turn them into custom CSS rules,
	 * which needs to be added to header.
	 */
	public function header_css() {
		// Rules to add
		$rules = '';

		/*
		 * Loop through the all groups of options.
		 *
		 * For example,
		 *
		 * $this->map = array(
		 *    // Group of colors with assigned array of types
		 *    'colors' => array(
		 *	     'primary' => array( ... ),
		 *	     'secondary' => array( ... ),
		 *    ),
		 *
		 *    // Group of fonts with assigned array of types
		 *    'fonts' => array(
		 *	     'primary' => array( ... ),
		 *		 'secondary' => array( ... ),
		 *	  ),
		 * );
		 */
		foreach ( $this->map as $group => $types ) {
			/*
			 * Loop through the all types.
			 *
			 * For example,
			 *
			 * array(
		     *     'primary' => array( ... ), // `Primary` type with data
		     *	   'secondary' => array( ... ), // `Secondary` type with data
		     * ),
			 *
			 */
			foreach ( $types as $key => $data ) {
				// Final loop, parse data to selector and value.
				foreach ( $data as $property => $selector ) {
					// Make the CSS rule, like so:
					// body { background-color: red; }
					$rules .= sprintf( '%s { %s: %s; }' . "\n", $selector, $property, $this->get_field_value( $key, $group ) );
				}
			}
		}

		// Add inline CSS to website <head>
		echo '<!--Customizer CSS--><style type="text/css">' . $rules . '</style><!--/Customizer CSS-->';
	}

	/**
	 * Bind color value with CSS selector for live updates in Customizer.
	 *
	 * @param string $option Option to bind
	 * @param string $selector Appropriate CSS selector
	 *
	 * @return string Resulting JS string for binding
	 */
	protected function js_bind_color( $option, $selector, $property ) {
		return sprintf( 'wp.customize( "%s", function( value ) {
									value.bind( function( to ) {
										$( "%s" ).css( "%s", to );
									} );
								} );', $option, $selector, $property );
	}

	/**
	 * Add JS script for binding colors to footer.
	 */
	public function footer_scripts() {

		$colors = '';
		// Loop through the all colors and bind them all.
		foreach ( $this->map['colors'] as $setting => $data ) {
			// Final loop, parse data to selector and value.
			foreach ( $data as $property => $selector ) {
				$colors .= $this->js_bind_color( sprintf( '%s_colors_%s', $this->settings_prefix, $setting ), $selector, $property );
			}
		}

		// Print script to footer
		printf( '<script type="text/javascript">( function( $ ) { %s } )( jQuery );</script>', $colors );
	}

	/**
	 * Helper method for making unique field names.
	 *
	 * @param string $key Key for setting
	 * @param string $group Setting group
	 *
	 * @return string Unique setting name
	 */
	protected function get_field_name( $key, $group ) {
		return sprintf( '%s_%s_%s', $this->settings_prefix, $group, $key );
	}

	/**
	 * Helper method for retrieving previously saved values.
	 * If value is not set yet, return default value.
	 *
	 * @param string $key Key for setting
	 * @param string $group Setting group
	 *
	 * @return string Saved or default value
	 */
	protected function get_field_value( $key, $group ) {
		// If option is set
		if ( $value = get_option( $this->settings_prefix.'_'.$group.'_'.$key ) ) {
			return $value;
		} else {
			// Return default value
			return $this->default_settings[ $group ][ $key ]['default'];
		}
	}
}

// Allow to override this class in child theme
if ( !class_exists( 'ventcamp_lite_Theme_Customizer' ) ) :
	/**
	 * Class that adds theme options to Customizer.
	 */
	class ventcamp_lite_Theme_Customizer extends Theme_Settings_Builder {
		/**
		 * ventcamp_lite_Theme_Customizer constructor.
		 */
		function __construct() {

			// Prefix before every setting
			$this->settings_prefix = 'ventcamp-lite';

			// Define default settings - colors and fonts
			$this->default_settings = array(
				'colors' => array(
					'accent' => array(
						'label' => __( 'Accent color', 'ventcamp-lite' ),
						'default' => '#fe4918',
					),
					'text' => array(
						'label' => __( 'Text color', 'ventcamp-lite' ),
						'default' => '#333333',
					),
					'heading' => array(
						'label' => __( 'Heading color', 'ventcamp-lite' ),
						'default' => '#333333',
					),
					'link' => array(
						'label' => __( 'Link color', 'ventcamp-lite' ),
						'default' => '#fe4918',
					),
					'link-hover' => array(
						'label' => __( 'Link hover color', 'ventcamp-lite' ),
						'default' => '#fe4918',
					),
				),

				'fonts' => array(
					'header' => array(
						'label' => __( 'Header font', 'ventcamp-lite' ),
						'default' => '"Lato", Arial, sans-serif',
					),
					'body' => array(
						'label' => __( 'Body font', 'ventcamp-lite' ),
						'default' => '"Helvetica Neue", Helvetica, Arial, sans-serif',
					),
					'menu' => array(
						'label' => __( 'Menu font', 'ventcamp-lite' ),
						'default' => '"Helvetica Neue", Helvetica, Arial, sans-serif',
					),
				)
			);

			/*
			 * Mapping of options with appropriate CSS selectors.
			 */
			$this->map = array(
				'colors' => array( //** $group => $types
					// Skin accent color
					'accent' => array( //** $type => $data
						// Text colors selectors
						'color' => '.woocommerce-message:before,' .
									'.woocommerce a.remove,' .
									'.woocommerce h1, .woocommerce h2, .woocommerce h3, .woocommerce h4, .woocommerce h5,' .
									'.footer .widget .widget-head:before,' .
									'.widget-area .widget .widget-head:before,' .
									'.post .entry-title a:hover,' .
									'.comment .fn a:hover,' .
									'.bypostauthor,' .
									'.tickera input.tickera-button,' .
									'.ticket-quantity .tickera_button,' .
									'#tickera_cart h2, #tickera_cart h3,' .
									'.ee-button-lnk.inline-button.ee-invoice-lnk,' .
									'.ee-spinner,' .
									'.vc-events-list .event-datetimes .ee-ical-sbmt,' .
									'label.error,' .
									'mark, .highlight,' .
									'.base-clr-txt,' .
									'.fa, .vc-oi, .typcn, .entypo-icon, .vc_li, .icon',

						// Background color selectors
						'background-color' => '.woocommerce a.remove:hover,' .
						                      '.tickera input.tickera-button:hover,' .
						                      '.tickera input.tickera-button.tickera_checkout,' .
						                      '.ee-button, .ee-button:active, .ee-button:hover, .ee-button:visited,' .
						                      '.ticket-selector-submit-btn, .spco-next-step-btn,' .
						                      '.base-clr-bg',

						// Border color selectors
						'border-color' => '.entry-navigation.paging .pagination > .current,' .
						                  '.tickera input.tickera-button,' .
						                  '.ee-button-lnk.inline-button.ee-invoice-lnk,' .
						                  '.ee-attention,' .
						                  '.vc-events-list article.espresso_events:hover,' .
						                  '.base-clr-bd',

						// Top border color selectors
						'border-top-color' => '.navigation-list > .menu-item-has-children > a:after'
					),
					'text' => array( //** $type => $data
						'color' => 'body,' .
						           '.woocommerce div.product p.price,' .
						           '.woocommerce div.product span.price,' .
						           '.tickera-checkout',
					),
					'heading' => array(  //** $type => $data
						'color' => 'h1, h2, h3, h4, h5, h6, .h7',
					),
					'link' => array(  //** $type => $data
						'color' => 'a',
					),
					'link-hover' => array(  //** $type => $data
						'color' => 'a:hover, a:active, a:focus',
					),
				),

				'fonts' => array(
					// Headings CSS selectors
					'header' => array(
						'font-family' => 'h1, h2, h3, h4, h5, h6, .h7, .heading-font,
										  .table > thead > tr > td,
										  .table > thead > tr > th,
										  table > thead > tr > td,
										  table > thead > tr > th,
										  .widget_tag_cloud .tagcloud li a,
										  .tickera th',
					),
					// Body text selectors
					'body' => array(
						'font-family' => 'body, .base-font, input, textarea, .tickera-checkout',
					),
					// Menu text selectors
					'menu' => array(
						'font-family' => '.navigation-list > li > a,
										  .navigation-list > li > ul > li > a,
										  .footer-nav > li a',
					),
				),
			);

			/*
			 * List of available font options.
			 */
			$this->fonts = array(
				'Arial' => 'Arial, sans-serif',
				'Helvetica' => '"Helvetica Neue", Helvetica, Arial, sans-serif',
				'Lato' => '"Lato", Arial, sans-serif',
				'Georgia' => 'Georgia, "Times New Roman", Times, serif',
			);

			$this->create();
		}
	}

	new ventcamp_lite_Theme_Customizer();
endif;